@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-search-dialog.c 5631 2007-06-17 15:07:49Z paolo $
+ * $Id: gedit-search-dialog.c 5931 2007-09-25 20:05:40Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -258,16 +258,30 @@ response_handler (GeditSearchDialog *dialog,
 		case GEDIT_SEARCH_DIALOG_REPLACE_ALL_RESPONSE:
 			str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->replace_text_entry));
 			if (*str != '\0')
+			{
+				gchar *text;
+
+				text = gedit_utils_unescape_search_text (str);
 				gedit_history_entry_prepend_text
 						(GEDIT_HISTORY_ENTRY (dialog->priv->replace_entry),
-						 str);
+						 text);
+
+				g_free (text);
+			}
 			/* fall through, so that we also save the find entry */
 		case GEDIT_SEARCH_DIALOG_FIND_RESPONSE:
 			str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->search_text_entry));
 			if (*str != '\0')
+			{
+				gchar *text;
+
+				text = gedit_utils_unescape_search_text (str);
 				gedit_history_entry_prepend_text
 						(GEDIT_HISTORY_ENTRY (dialog->priv->search_entry),
-						 str);
+						 text);
+
+				g_free (text);
+			}
 	}
 }
 
@@ -317,6 +331,12 @@ gedit_search_dialog_init (GeditSearchDialog *dlg)
 	gtk_dialog_add_buttons (GTK_DIALOG (dlg),
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 				NULL);
+				
+	/* HIG defaults */
+	gtk_container_set_border_width (GTK_CONTAINER (dlg), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 2); /* 2 * 5 + 2 = 12 */
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->action_area), 6);
 
 	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "gedit-search-dialog.glade",
 					     "search_dialog_content",
@@ -337,6 +357,7 @@ gedit_search_dialog_init (GeditSearchDialog *dlg)
 
 		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 					     error_widget);
+		gtk_container_set_border_width (GTK_CONTAINER (error_widget), 5);					     
 
 		dlg->priv->glade_error = TRUE;
 
@@ -346,6 +367,10 @@ gedit_search_dialog_init (GeditSearchDialog *dlg)
 	dlg->priv->search_entry = gedit_history_entry_new ("gedit2_search_for_entry",
 							   TRUE);
 	gtk_widget_set_size_request (dlg->priv->search_entry, 300, -1);
+	gedit_history_entry_set_escape_func
+			(GEDIT_HISTORY_ENTRY (dlg->priv->search_entry),
+			 (GeditHistoryEntryEscapeFunc) gedit_utils_escape_search_text);
+
 	dlg->priv->search_text_entry = gedit_history_entry_get_entry
 			(GEDIT_HISTORY_ENTRY (dlg->priv->search_entry));
 	gtk_entry_set_activates_default (GTK_ENTRY (dlg->priv->search_text_entry),
@@ -357,6 +382,10 @@ gedit_search_dialog_init (GeditSearchDialog *dlg)
 
 	dlg->priv->replace_entry = gedit_history_entry_new ("gedit2_replace_with_entry",
 							    TRUE);
+	gedit_history_entry_set_escape_func
+			(GEDIT_HISTORY_ENTRY (dlg->priv->replace_entry),
+			 (GeditHistoryEntryEscapeFunc) gedit_utils_escape_search_text);
+
 	dlg->priv->replace_text_entry = gedit_history_entry_get_entry
 			(GEDIT_HISTORY_ENTRY (dlg->priv->replace_entry));
 	gtk_entry_set_activates_default (GTK_ENTRY (dlg->priv->replace_text_entry),
@@ -405,6 +434,7 @@ gedit_search_dialog_init (GeditSearchDialog *dlg)
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 			    content, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (content), 5);	     
 
 	g_signal_connect (dlg->priv->search_text_entry,
 			  "insert_text",
