@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-app.h 5666 2007-06-29 19:52:25Z sfre $
+ * $Id: gedit-app.h 5988 2007-10-28 14:44:53Z sfre $
  */
 
 #ifndef __GEDIT_APP_H__
@@ -112,9 +112,11 @@ GeditLockdownMask gedit_app_get_lockdown		(GeditApp *app);
  */
 GeditWindow	*_gedit_app_restore_window		(GeditApp    *app,
 							 const gchar *role);
-GeditWindow	*_gedit_app_get_window_in_workspace	(GeditApp  *app,
-							 GdkScreen *screen,
-							 gint       workspace);
+GeditWindow	*_gedit_app_get_window_in_viewport	(GeditApp     *app,
+							 GdkScreen    *screen,
+							 gint          workspace,
+							 gint          viewport_x,
+							 gint          viewport_y);
 void		 _gedit_app_set_lockdown		(GeditApp          *app,
 							 GeditLockdownMask  lockdown);
 void		 _gedit_app_set_lockdown_bit		(GeditApp          *app,
