@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c 5812 2007-08-15 12:19:59Z pborelli $
+ * $Id: gedit-commands-file.c 5993 2007-11-02 14:30:31Z sfre $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -301,17 +301,18 @@ gedit_commands_load_uri (GeditWindow         *window,
 			 gint                 line_pos)
 {
 	GSList *uris = NULL;
-	
+
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
 	g_return_if_fail (uri != NULL);
+	g_return_if_fail (gedit_utils_is_valid_uri (uri));
+
+	gedit_debug_message (DEBUG_COMMANDS, "Loading URI '%s'", uri);
 
-	gedit_debug (DEBUG_COMMANDS);	
-	
 	uris = g_slist_prepend (uris, (gchar *)uri);
-	
+
 	load_file_list (window, uris, encoding, line_pos, FALSE);
-	
-	g_slist_free (uris);	
+
+	g_slist_free (uris);
 }
 
 /**
