@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c 5756 2007-07-29 17:30:40Z sfre $
+ * $Id: gedit-document.c 6001 2007-11-04 17:33:27Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +47,7 @@
 #include "gedit-utils.h"
 #include "gedit-metadata-manager.h"
 #include "gedit-language-manager.h"
-#include "gedit-source-style-manager.h"
+#include "gedit-style-scheme-manager.h"
 #include "gedit-document-loader.h"
 #include "gedit-document-saver.h"
 #include "gedit-marshal.h"
@@ -98,6 +98,7 @@ struct _GeditDocumentPrivate
 	gint	     last_save_was_manually : 1; 	
 	gint	     language_set_by_user : 1;
 	gint         stop_cursor_moved_emission : 1;
+	gint         dispose_has_run : 1;
 
 	gchar	    *uri;
 	gint 	     untitled_number;
@@ -205,19 +206,17 @@ release_untitled_number (gint n)
 }
 
 static void
-gedit_document_finalize (GObject *object)
+gedit_document_dispose (GObject *object)
 {
 	GeditDocument *doc = GEDIT_DOCUMENT (object); 
-	
+
 	gedit_debug (DEBUG_DOCUMENT);
 
-	if (doc->priv->untitled_number > 0)
-	{
-		g_return_if_fail (doc->priv->uri == NULL);
-		release_untitled_number (doc->priv->untitled_number);
-	}
-
-	if (doc->priv->uri != NULL)
+	/* Metadata must be saved here and not in finalize
+	 * because the language is gone by the time finalize runs.
+	 * beside if some plugin prevents proper finalization by
+	 * holding a ref to the doc, we still save the metadata */
+	if ((!doc->priv->dispose_has_run) && (doc->priv->uri != NULL))
 	{
 		GtkTextIter iter;
 		gchar *position;
@@ -247,17 +246,37 @@ gedit_document_finalize (GObject *object)
 		}
 	}
 
+	if (doc->priv->loader)
+	{
+		g_object_unref (doc->priv->loader);
+		doc->priv->loader = NULL;
+	}
+
+	doc->priv->dispose_has_run = TRUE;
+
+	G_OBJECT_CLASS (gedit_document_parent_class)->dispose (object);
+}
+
+static void
+gedit_document_finalize (GObject *object)
+{
+	GeditDocument *doc = GEDIT_DOCUMENT (object); 
+
+	gedit_debug (DEBUG_DOCUMENT);
+
+	if (doc->priv->untitled_number > 0)
+	{
+		g_return_if_fail (doc->priv->uri == NULL);
+		release_untitled_number (doc->priv->untitled_number);
+	}
+
 	g_free (doc->priv->uri);
 	if (doc->priv->vfs_uri != NULL)
 		gnome_vfs_uri_unref (doc->priv->vfs_uri);
 
 	g_free (doc->priv->mime_type);
-
-	if (doc->priv->loader)
-		g_object_unref (doc->priv->loader);
-
 	g_free (doc->priv->search_text);
-	
+
 	if (doc->priv->to_search_region != NULL)
 	{
 		/* we can't delete marks if we're finalizing the buffer */
@@ -366,6 +385,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkTextBufferClass *buf_class = GTK_TEXT_BUFFER_CLASS (klass);
 
+	object_class->dispose = gedit_document_dispose;
 	object_class->finalize = gedit_document_finalize;
 	object_class->get_property = gedit_document_get_property;
 	object_class->set_property = gedit_document_set_property;	
@@ -539,11 +559,11 @@ set_language (GeditDocument     *doc,
 	gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (doc), lang);
 
 	if (lang != NULL)
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc),
-						 gedit_prefs_manager_get_enable_syntax_highlighting ());
+		gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (doc),
+				 gedit_prefs_manager_get_enable_syntax_highlighting ());
 	else
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc), 
-						 FALSE);
+		gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (doc), 
+				 FALSE);
 
 	if (set_by_user && (doc->priv->uri != NULL))
 	{
@@ -583,10 +603,36 @@ set_encoding (GeditDocument       *doc,
 	g_object_notify (G_OBJECT (doc), "encoding");
 }
 
+static GtkSourceStyleScheme *
+get_default_style_scheme (void)
+{
+	gchar *scheme_id;
+	GtkSourceStyleScheme *def_style;
+	GtkSourceStyleSchemeManager *manager;
+
+	manager = gedit_get_style_scheme_manager ();
+	scheme_id = gedit_prefs_manager_get_source_style_scheme ();
+	def_style = gtk_source_style_scheme_manager_get_scheme (manager,
+								scheme_id);
+
+	if (def_style == NULL)
+	{
+		g_warning ("Default style scheme '%s' cannot be found, falling back to 'classic' style scheme ", scheme_id);
+
+		def_style = gtk_source_style_scheme_manager_get_scheme (manager, "classic");
+		if (def_style == NULL) 
+		{
+			g_warning ("Style scheme 'classic' cannot be found, check your GtkSourceView installation.");			
+		}
+	}
+
+	g_free (scheme_id);
+
+	return def_style;
+}
 static void
 gedit_document_init (GeditDocument *doc)
 {
-	GtkSourceStyleManager *style_manager;
 	GtkSourceStyleScheme *style_scheme;
 
 	gedit_debug (DEBUG_DOCUMENT);
@@ -606,6 +652,8 @@ gedit_document_init (GeditDocument *doc)
 	doc->priv->last_save_was_manually = TRUE;
 	doc->priv->language_set_by_user = FALSE;
 
+	doc->priv->dispose_has_run = FALSE;
+
 	doc->priv->mtime = 0;
 
 	g_get_current_time (&doc->priv->time_of_last_save_or_load);
@@ -615,15 +663,13 @@ gedit_document_init (GeditDocument *doc)
 	gtk_source_buffer_set_max_undo_levels (GTK_SOURCE_BUFFER (doc), 
 					       gedit_prefs_manager_get_undo_actions_limit ());
 
-	/* TODO: Set the bracket matching tag style -- Paolo (10 Jan. 2005) */
-	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (doc), 
-					      gedit_prefs_manager_get_bracket_matching ());
+	gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (doc), 
+							   gedit_prefs_manager_get_bracket_matching ());
 
 	gedit_document_set_enable_search_highlighting (doc,
 						       gedit_prefs_manager_get_enable_search_highlighting ());
 
-	style_manager = gedit_get_source_style_manager ();
-	style_scheme = gedit_source_style_manager_get_default_scheme (style_manager);
+	style_scheme = get_default_style_scheme ();
 	if (style_scheme != NULL)
 		gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc),
 						    style_scheme);
@@ -724,7 +770,7 @@ set_uri (GeditDocument *doc,
 
 			if (strcmp (data, "_NORMAL_") != 0)
 			{
-				language = gtk_source_language_manager_get_language_by_id (
+				language = gtk_source_language_manager_get_language (
 							gedit_get_language_manager (),
 							data);
 			}
@@ -918,7 +964,7 @@ _gedit_document_check_externally_modified (GeditDocument *doc)
 {
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult result;
-	gint file_mtime;
+	gboolean res = FALSE;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
@@ -940,11 +986,14 @@ _gedit_document_check_externally_modified (GeditDocument *doc)
 		return FALSE;
 	}
 
-	file_mtime = info->mtime;
+	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
+	{
+		res = info->mtime > doc->priv->mtime;
+	}
 
 	gnome_vfs_file_info_unref (info);
 
-	return (file_mtime > doc->priv->mtime);
+	return res;
 }
 
 static void
@@ -1615,7 +1664,7 @@ gedit_document_replace_all (GeditDocument       *doc,
 	gchar *replace_text;
 	gint replace_text_len;
 	GtkTextBuffer *buffer;
-	gboolean check_brackets;
+	gboolean brackets_highlighting;
 	gboolean search_highliting;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), 0);
@@ -1649,8 +1698,8 @@ gedit_document_replace_all (GeditDocument       *doc,
 	doc->priv->stop_cursor_moved_emission = TRUE;
 
 	/* also avoid spending time matching brackets */
-	check_brackets = gtk_source_buffer_get_check_brackets (GTK_SOURCE_BUFFER (buffer));
-	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (buffer), FALSE);
+	brackets_highlighting = gtk_source_buffer_get_highlight_matching_brackets (GTK_SOURCE_BUFFER (buffer));
+	gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (buffer), FALSE);
 
 	/* and do search highliting later */
 	search_highliting = gedit_document_get_enable_search_highlighting (doc);
@@ -1706,8 +1755,8 @@ gedit_document_replace_all (GeditDocument       *doc,
 	doc->priv->stop_cursor_moved_emission = FALSE;
 	emit_cursor_moved (doc);
 
-	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (buffer),
-					      check_brackets);
+	gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (buffer),
+							   brackets_highlighting);
 	gedit_document_set_enable_search_highlighting (doc, search_highliting);
 
 	g_free (search_text);
@@ -1755,6 +1804,110 @@ _gedit_document_get_seconds_since_last_save_or_load (GeditDocument *doc)
 	return (current_time.tv_sec - doc->priv->time_of_last_save_or_load.tv_sec);
 }
 
+static void
+get_search_match_colors (GeditDocument *doc,
+			 gboolean      *foreground_set,
+			 GdkColor      *foreground,
+			 gboolean      *background_set,
+			 GdkColor      *background)
+{
+	GtkSourceStyleScheme *style_scheme;
+	GtkSourceStyle *style;
+	gchar *bg;
+	gchar *fg;
+
+	style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (doc));
+	if (style_scheme == NULL)
+		goto fallback;
+
+	style = gtk_source_style_scheme_get_style (style_scheme,
+						   "search-match");
+	if (style == NULL)
+		goto fallback;
+
+	g_object_get (style, 
+		      "foreground-set", foreground_set, 
+		      "foreground", &fg,
+		      "background-set", background_set, 
+		      "background", &bg,
+		      NULL);
+
+	if (*foreground_set)
+	{
+		if (fg == NULL ||
+		    !gdk_color_parse (fg, foreground))
+		{
+			*foreground_set = FALSE;
+		}
+	}
+
+	if (*background_set)
+	{
+		if (bg == NULL ||
+		    !gdk_color_parse (bg, background))
+		{
+			*background_set = FALSE;
+		}
+	}	
+
+	g_free (fg);
+	g_free (bg);
+
+	return;
+
+ fallback:
+	gedit_debug_message (DEBUG_DOCUMENT, 
+			     "Falling back to hard-coded colors "
+			     "for the \"found\" text tag.");
+
+	gdk_color_parse ("#FFFF78", background);
+	*background_set = TRUE;
+	*foreground_set = FALSE;
+
+	return;
+}
+
+static void
+sync_found_tag (GeditDocument *doc,
+		GParamSpec    *pspec,
+		gpointer       data)
+{
+	GdkColor fg;
+	GdkColor bg;
+	gboolean fg_set;
+	gboolean bg_set;
+
+	gedit_debug (DEBUG_DOCUMENT);
+
+	g_return_if_fail (GTK_TEXT_TAG (doc->priv->found_tag));
+
+	get_search_match_colors (doc,
+				 &fg_set, &fg,
+				 &bg_set, &bg);
+
+	if (fg_set)
+		g_object_set (doc->priv->found_tag,
+			      "foreground-gdk", &fg,
+			      NULL);
+
+	if (bg_set)
+		g_object_set (doc->priv->found_tag,
+			      "background-gdk", &bg,
+			      NULL);
+}
+
+static void
+text_tag_set_highest_priority (GtkTextTag    *tag,
+			       GtkTextBuffer *buffer)
+{
+	GtkTextTagTable *table;
+	gint n;
+
+	table = gtk_text_buffer_get_tag_table (buffer);
+	n = gtk_text_tag_table_get_size (table);
+	gtk_text_tag_set_priority (tag, n - 1);
+}
+
 static void
 search_region (GeditDocument *doc,
 	       GtkTextIter   *start,
@@ -1774,14 +1927,24 @@ search_region (GeditDocument *doc,
 	
 	if (doc->priv->found_tag == NULL)
 	{
-		/* FIXME: the colors are hardcoded */
 		doc->priv->found_tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (doc), 
 								   "found",
-								   "background", "#FFFF78",
-								   //"foreground", "black",
 								   NULL);
+
+		sync_found_tag (doc, NULL, NULL);
+
+		g_signal_connect (doc, 
+				  "notify::style-scheme",
+				  G_CALLBACK (sync_found_tag),
+				  NULL);
 	}
 
+	/* make sure the 'found' tag has the priority over
+	 * syntax highlighting tags */
+	text_tag_set_highest_priority (doc->priv->found_tag,
+				       GTK_TEXT_BUFFER (doc));
+				   
+
 	if (doc->priv->search_text == NULL)
 		return;
 		
