@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-panel.c 5244 2006-09-23 14:37:09Z pborelli $
+ * $Id: gedit-panel.c 5796 2007-08-14 17:26:09Z pborelli $
  */
 
 #include "gedit-panel.h"
@@ -52,8 +52,6 @@ struct _GeditPanelPrivate
 
 	/* Notebook */
 	GtkWidget *notebook;
-
-	GtkTooltips *tooltips;
 };
 
 typedef struct _GeditPanelItem GeditPanelItem;
@@ -100,8 +98,6 @@ G_DEFINE_TYPE(GeditPanel, gedit_panel, GTK_TYPE_VBOX)
 static void
 gedit_panel_finalize (GObject *obj)
 {
-	g_object_unref (GEDIT_PANEL (obj)->priv->tooltips);
-
 	if (G_OBJECT_CLASS (gedit_panel_parent_class)->finalize)
 		(*G_OBJECT_CLASS (gedit_panel_parent_class)->finalize) (obj);
 }
@@ -416,9 +412,6 @@ gedit_panel_init (GeditPanel *panel)
 {
 	panel->priv = GEDIT_PANEL_GET_PRIVATE (panel);
 	g_return_if_fail (panel->priv != NULL);	
-
-	panel->priv->tooltips = gtk_tooltips_new ();
-	g_object_ref_sink (panel->priv->tooltips);
 }
 
 static void
@@ -518,12 +511,9 @@ build_horizontal_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      close_button,
-			      _("Hide panel"),
-			      NULL);
+	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
 
-	g_signal_connect (G_OBJECT (close_button),
+	g_signal_connect (close_button,
 			  "clicked",
                           G_CALLBACK (close_button_clicked_cb),
                           panel);
@@ -595,16 +585,13 @@ build_vertical_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      close_button,
-			      _("Hide panel"),
-			      NULL);
+	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
 
-	g_signal_connect (G_OBJECT (close_button), 
+	g_signal_connect (close_button,
 			  "clicked",
                           G_CALLBACK (close_button_clicked_cb),
                           panel);
-                 
+
 	gtk_widget_show_all (title_hbox);
 
 	gtk_box_pack_start (GTK_BOX (panel),
@@ -681,10 +668,7 @@ build_tab_label (GeditPanel  *panel,
         gtk_misc_set_padding (GTK_MISC (label), 0, 0);
 	gtk_box_pack_start (GTK_BOX (label_hbox), label, TRUE, TRUE, 0);
 
-	gtk_tooltips_set_tip (panel->priv->tooltips,
-			      label_ebox,
-			      name,
-			      NULL);
+	gtk_widget_set_tooltip_text (label_ebox, name);
 
 	gtk_widget_show_all (hbox);
 
@@ -798,10 +782,6 @@ gedit_panel_remove_item (GeditPanel *panel,
 		           NULL);
 
 	ebox = g_object_get_data (G_OBJECT (item), "label-ebox");
-	gtk_tooltips_set_tip (GTK_TOOLTIPS (panel->priv->tooltips), 
-			      ebox, 
-			      NULL, 
-			      NULL);  
 	
 	/* ref the item to keep it alive during signal emission */
 	g_object_ref (G_OBJECT (item));
