@@ -42,6 +42,7 @@
 #include "gedit-utils.h"
 #include "gedit-message-area.h"
 #include "gedit-io-error-message-area.h"
+#include "gedit-print.h"
 #include "gedit-print-job-preview.h"
 #include "gedit-progress-message-area.h"
 #include "gedit-debug.h"
@@ -2098,18 +2099,38 @@ _gedit_tab_save_as (GeditTab            *tab,
 		    const GeditEncoding *encoding)
 {
 	GeditDocument *doc;
+	GeditDocumentSaveFlags save_flags;
 
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_NORMAL) ||
+			  (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION) ||
 			  (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW));
 	g_return_if_fail (encoding != NULL);
-				  
+
 	g_return_if_fail (tab->priv->tmp_save_uri == NULL);
 	g_return_if_fail (tab->priv->tmp_encoding == NULL);
-	
+
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
+	/* reset the save flags, when saving as */
+	tab->priv->save_flags = 0;
+
+	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
+	{
+		/* We already told the user about the external
+		 * modification: hide the message area and set
+		 * the save flag.
+		 */
+
+		set_message_area (tab, NULL);
+		save_flags = tab->priv->save_flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME;
+	}
+	else
+	{
+		save_flags = tab->priv->save_flags;
+	}
+
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SAVING);
 
 	/* uri used in error messages... strdup because errors are async
@@ -2117,9 +2138,6 @@ _gedit_tab_save_as (GeditTab            *tab,
 	tab->priv->tmp_save_uri = g_strdup (uri);
 	tab->priv->tmp_encoding = encoding;
 
-	/* reset the save flags, when saving as */
-	tab->priv->save_flags = 0;
-	
 	if (tab->priv->auto_save_timeout > 0)
 		remove_auto_save_timeout (tab);
 
@@ -2131,10 +2149,15 @@ print_preview_destroyed (GtkWidget *preview,
 			 GeditTab  *tab)
 {
 	tab->priv->print_preview = NULL;
-	
+
 	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
 	{
+		GeditView *view;
+
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+
+		view = gedit_tab_get_view (tab);
+		gtk_widget_grab_focus (GTK_WIDGET (view));
 	}
 	else
 	{
@@ -2230,10 +2253,12 @@ static void
 print_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 {
 	GnomePrintJob *gjob;
+	GeditView *view;
 
 	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+
 	set_message_area (tab, NULL); /* destroy the message area */
-	
+
 	gjob = gtk_source_print_job_get_print_job (pjob);
 
 	gnome_print_job_print (gjob);
@@ -2250,8 +2275,11 @@ print_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 		gtk_widget_destroy (tab->priv->print_preview);
 		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
 	}
-	
+
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+
+	view = gedit_tab_get_view (tab);
+	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 static void
@@ -2259,6 +2287,8 @@ print_cancelled (GeditMessageArea *area,
                  gint              response_id,
                  GeditTab         *tab)
 {
+	GeditView *view;
+
 	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
 	
 	gtk_source_print_job_cancel (GTK_SOURCE_PRINT_JOB (tab->priv->print_job));
@@ -2273,8 +2303,11 @@ print_cancelled (GeditMessageArea *area,
 		gtk_widget_destroy (tab->priv->print_preview);
 		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
 	}
-	
+
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+
+	view = gedit_tab_get_view (tab);
+	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 static void
@@ -2303,12 +2336,15 @@ show_printing_message_area (GeditTab      *tab,
 
 void 
 _gedit_tab_print (GeditTab      *tab,
-		  GeditPrintJob *pjob,
+		  gpointer       print_job,
 		  GtkTextIter   *start, 
 		  GtkTextIter   *end)
 {
+	GeditPrintJob *pjob;
 	GeditDocument *doc;
-	
+
+	pjob = (GeditPrintJob *) print_job;
+
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
 	g_return_if_fail (start != NULL);
@@ -2348,12 +2384,15 @@ _gedit_tab_print (GeditTab      *tab,
 
 void
 _gedit_tab_print_preview (GeditTab      *tab,
-			  GeditPrintJob *pjob,
+			  gpointer       print_job,
 			  GtkTextIter   *start, 
-			  GtkTextIter   *end)		  
+			  GtkTextIter   *end)
 {
+	GeditPrintJob *pjob;
 	GeditDocument *doc;
-	
+
+	pjob = (GeditPrintJob *) print_job;
+
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
 	g_return_if_fail (start != NULL);
