@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-tab.h,v 1.3 2006/01/19 18:16:00 paolo Exp $
+ * $Id: gedit-tab.h 5667 2007-06-30 09:51:18Z pborelli $
  */
 
 #ifndef __GEDIT_TAB_H__
@@ -35,7 +35,6 @@
 
 #include <gedit/gedit-view.h>
 #include <gedit/gedit-document.h>
-#include <gedit/gedit-print.h>
 
 G_BEGIN_DECLS
 
@@ -54,6 +53,7 @@ typedef enum
 	GEDIT_TAB_STATE_SAVING_ERROR,
 	GEDIT_TAB_STATE_GENERIC_ERROR,
 	GEDIT_TAB_STATE_CLOSING,
+	GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION,
 	GEDIT_TAB_NUM_OF_STATES /* This is not a valid state */
 } GeditTabState;
 
@@ -62,7 +62,6 @@ typedef enum
  */
 #define GEDIT_TYPE_TAB              (gedit_tab_get_type())
 #define GEDIT_TAB(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_TAB, GeditTab))
-#define GEDIT_TAB_CONST(obj)        (G_TYPE_CHECK_INSTANCE_CAST((obj), GEDIT_TYPE_TAB, GeditTab const))
 #define GEDIT_TAB_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST((klass), GEDIT_TYPE_TAB, GeditTabClass))
 #define GEDIT_IS_TAB(obj)           (G_TYPE_CHECK_INSTANCE_TYPE((obj), GEDIT_TYPE_TAB))
 #define GEDIT_IS_TAB_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GEDIT_TYPE_TAB))
@@ -136,7 +135,7 @@ GtkWidget	*_gedit_tab_new_from_uri	(const gchar         *uri,
 gchar 		*_gedit_tab_get_name		(GeditTab            *tab);
 gchar 		*_gedit_tab_get_tooltips	(GeditTab            *tab);
 GdkPixbuf 	*_gedit_tab_get_icon		(GeditTab            *tab);
-gboolean	 _gedit_tab_load		(GeditTab            *tab,
+void		 _gedit_tab_load		(GeditTab            *tab,
 						 const gchar         *uri,
 						 const GeditEncoding *encoding,
 						 gint                 line_pos,
@@ -146,12 +145,15 @@ void		 _gedit_tab_save		(GeditTab            *tab);
 void		 _gedit_tab_save_as		(GeditTab            *tab,
 						 const gchar         *uri,
 						 const GeditEncoding *encoding);
+
+/* print_job is a gpointer instead of GeditPrintJob because gedit-print is
+ * is not public so it cannot be included in this header */
 void		 _gedit_tab_print		(GeditTab            *tab,
-						 GeditPrintJob       *pjob,
+						 gpointer             print_job,
 						 GtkTextIter         *start, 
 			  			 GtkTextIter         *end);
 void		 _gedit_tab_print_preview	(GeditTab            *tab,
-						 GeditPrintJob       *pjob,
+						 gpointer             print_job,
 						 GtkTextIter         *start, 
 			  			 GtkTextIter         *end);
 void		 _gedit_tab_mark_for_closing	(GeditTab	     *tab);
